#pragma once
#include "ShaderProgram.h"
#include "Shader.h"
#include "Mesh.h"
#include "Slider.h"
#include "EnvironmentMap.h"
#include "SongPlayer.h"
#include <string>
#include <vector>
struct Material;
class Texture;
class SubMesh;

class AssetManager
{
public:
	AssetManager();
	~AssetManager();
	static AssetManager* GetInstance() {return s_instance;}
	std::string GetDataPath() {return "data/";}
	void LoadAll();
	void FreeAll();
	Texture* LoadTexture(std::string fileName);
	Texture* LoadCubemapTexture(std::string prefix, std::string extension);
	Mesh* LoadMesh(std::string fileName, MeshFlags flags);
	void LoadMtllib(std::string fileName);
	Texture* GetTexture(std::string name, bool assertOnFail=true);
	Mesh* GetMesh(std::string name, bool assertOnFail=true);
	Material GetMaterial(std::string name, bool assertOnFail=true);
	void UseProgram(ShaderProgram* p);
	ShaderProgram* GetCurrentProgram() {return m_currentProg;}
	ShaderProgram* GetNonInstanceProg() {return &m_nonInstanceShaderProg;}
	ShaderProgram* GetInstanceProg() {return &m_instanceShaderProg;}
	ShaderProgram* GetCubemapProg() {return &m_cubemapShaderProg;}
	ShaderProgram* GetPostProcessingProg() {return &m_postProcessProg;}
	ShaderProgram* GetDeformPostProcProg() {return &m_deformPostProcProg;}
	EnvironmentMap* GetEnvironmentMap() {return &m_environmentMap;}
	SongPlayer* GetSongPlayer() {return &m_songPlayer;}
	Texture* GetOffscreenTexture(float width, float height);
	unsigned int GetFbo() {return m_fbo;}
	Texture* GetDepthBufferTex(unsigned int width, unsigned int height) 
	{
		if(!m_depthbuffer.IsLoaded())
		{
			m_depthbuffer.CreateDepth(width, height);
		}
		else if(m_depthbuffer.GetWidth()!=width || m_depthbuffer.GetHeight()!=height) {
			m_depthbuffer.Free();
			m_depthbuffer.CreateDepth(width, height);
		}
		return &m_depthbuffer;
	}
	unsigned int GetDepthBufferRB()
	{
		return m_depthBufferRB;
	}
	
	
	struct Parameter
	{
		std::string m_name;
		void* data;
		
		unsigned int lo;
		unsigned int hi;
	};
	void AddParameter(std::string name, unsigned int* i, unsigned int lo, unsigned int hi);
	void AddParameter(std::string name, float* f);
	unsigned int GetNumSliders() {return m_params.size();}
	Slider& GetSlider(unsigned int index) {return m_params[index];}
private:
	std::vector<Slider> m_params;
	std::vector<Texture*> m_textures;
	std::vector<Mesh*> m_meshes;
	std::vector<Material> m_materials;
	static AssetManager* s_instance;

	EnvironmentMap m_environmentMap;
	Texture m_offscreenTex;
	ShaderProgram m_instanceShaderProg;
	ShaderProgram m_nonInstanceShaderProg;
	ShaderProgram m_cubemapShaderProg; // For sampling from env map
	ShaderProgram m_postProcessProg;
	ShaderProgram m_deformPostProcProg; // For explosions
	ShaderProgram* m_currentProg;
	SongPlayer m_songPlayer;
	Shader m_postProcessFS;
	Shader m_instancedVS, m_defaultFS;
	Shader m_simpleVS, m_cubeMapFS;
	bool m_offscrTexLoaded;

	unsigned int m_fbo;
	unsigned int m_depthBufferRB;
	Texture m_depthbuffer;
};