#include "stdafx.h"
#include "EnemyList.h"
#include "AssetManager.h"
#include "transform.h"
#include "Random.h"
#include "SubMesh.h"

void EnemyList::Init()
{
#if LOWPOLY
	m_mesh = AssetManager::GetInstance()->GetMesh("enemy2_lowpoly.obj");
#else
	m_mesh = AssetManager::GetInstance()->GetMesh("enemy2.obj");
#endif
}

static const float ENEMY_SPEED = .15;

void EnemyList::AddEnemy(Point3D worldPos)
{
	if(m_enemies.size() == SubMesh::MAX_INSTANCES) return;
	Enemy e;
	e.pos = worldPos;
	e.velocity = Vector3D(RandFrac()*ENEMY_SPEED, RandFrac()*ENEMY_SPEED, -RandUnsignedFrac()*10*ENEMY_SPEED);
	e.transform = translation(Vector3D(worldPos[0], worldPos[1], worldPos[2]));
	m_enemies.push_back(e);
}

bool EnemyList::isDead(const Enemy& e)
{
	return e.dead;
}

void EnemyList::Update(Point3D cameraPos, Matrix4x4 cameraRot, MissileList& missiles,  std::list<Explosion>& explosions, unsigned int& score, bool gameLogic)
{
	if(!m_mesh) return;
	// Update positions
	int i = 0;
	for(std::list<Enemy>::iterator iter = m_enemies.begin(); iter!=m_enemies.end(); iter++)  {
		if(gameLogic) iter->pos += iter->velocity;
		Vector3D transl = cameraPos - iter->pos;
		//iter->transform = cameraRot * translation(transl);
		iter->transform = translation(transl);
		m_mesh->SetInstancing(iter->transform, i);
		i++;
	}
	if(!gameLogic) return;

	for(std::list<Enemy>::iterator iter = m_enemies.begin(); iter!=m_enemies.end(); iter++)  {
		if(missiles.CollidedWith(iter->pos, m_mesh->GetBoundingBox())) {
			iter->dead = true;
			score += 10;
			Explosion e;
			e.Create(iter->pos);
			explosions.push_back(e);
		}			
	}

	// Prune missiles that are out of bounds
	m_enemies.remove_if(EnemyList::isDead);
}

void EnemyList::Draw()
{
	m_mesh->Draw(m_enemies.size());
}