#include "stdafx.h"
#include "FractalScript.h"
#include "AssetManager.h"
#include "Random.h"
#include "GlobalSettings.h"


FractalScript::Line::Line()
{
	useDeform = false;
	looping = true;
	numChildren = 0;
}

FractalScript::Line::Line( Mesh* m )
{
	useDeform = false;
	looping = true;
	numChildren = 0;
	meshes.push_back(m);
}

FractalScript::Line::Line( Mesh* m[], int n )
{
	useDeform = false;
	looping = true;
	numChildren = 0;
	for(int i=0; i<n; i++) meshes.push_back(m[i]);
}


void FractalScript::GenerateRandom()
{


	Line l1(AssetManager::GetInstance()->GetMesh("stem.obj"));
	l1.placementScheme = CENTER;
	l1.numChildren = RandInt(2, 5*g_globalSettings.density);
	l1.rotateIncr = Vector3D(RandFrac()*0.001, RandFrac()*1, RandFrac()*0.001);
	m_lines.push_back(l1);

	int numLevels = RandInt(1, 5*g_globalSettings.density);
	for(int i=0; i<numLevels; i++) {
		Line l(AssetManager::GetInstance()->GetMesh("stem.obj"));
		l.placementScheme = TREE;
		l.numChildren = RandInt(4, 10.0*g_globalSettings.density);
		l.rotateIncr = Vector3D(RandFrac()*0.001, RandFrac()*2, RandFrac()*0.001);
		if(i==0)l.translateIncr = Vector3D(0, 0, -5);
		m_lines.push_back(l);
	}

	Line l(AssetManager::GetInstance()->GetMesh("stem.obj"));
	l.placementScheme = TREE;
	l.numChildren = 0;
	l.rotateIncr = Vector3D(RandFrac()*0.001, RandFrac()*2, RandFrac()*0.001);
	m_lines.push_back(l);

}

void FractalScript::ScriptCases()
{
	if(0) /* a flower */
	{

		Line l1(AssetManager::GetInstance()->GetMesh("stem.obj"));
		l1.placementScheme = CENTER;
		l1.numChildren = RandInt(2, 5*g_globalSettings.density);
		l1.rotateIncr = Vector3D(RandFrac()*0.001, RandFrac()*1, RandFrac()*0.001);
		m_lines.push_back(l1);

		int numLevels = RandInt(1, 5*g_globalSettings.density);
		for(int i=0; i<numLevels; i++) {
			Line l(AssetManager::GetInstance()->GetMesh("stem.obj"));
			l.placementScheme = TREE;
			l.numChildren = RandInt(4, 10.0*g_globalSettings.density);
			l.rotateIncr = Vector3D(RandFrac()*0.001, RandFrac()*2, RandFrac()*0.001);
			if(i==0)l.translateIncr = Vector3D(0, 0, -5);
			m_lines.push_back(l);
		}

		Line l(AssetManager::GetInstance()->GetMesh("stem.obj"));
		l.placementScheme = TREE;
		l.numChildren = RandInt(7, 20.0*g_globalSettings.density);
		l.rotateIncr = Vector3D(RandFrac()*0.001, RandFrac()*2, RandFrac()*0.001);
		m_lines.push_back(l);

		Mesh* meshes[] = {AssetManager::GetInstance()->GetMesh("petal3a.obj")};
		Line l3(meshes, 1);
		l3.placementScheme = SPIKE;
		l3.numChildren = 0;
		l3.useDeform = true;
		m_lines.push_back(l3);
	}
	else if(0){ // buildings

		Line l;
		l.placementScheme = CENTER;
		l.numChildren = 100;
		m_lines.push_back(l);

		Mesh* meshes[] = {AssetManager::GetInstance()->GetMesh("building1.obj")};
		Line l2(meshes, 1);
		l2.placementScheme = XZFIELD;
		l2.numChildren = 0;
		m_lines.push_back(l2);
	}
	else if(0)
	{

		// a starfield
		Line l1;
		l1.placementScheme = CENTER;
		l1.numChildren = 50;
		m_lines.push_back(l1);

		Line l2(AssetManager::GetInstance()->GetMesh("star.obj"));
		l2.placementScheme = XYFIELD;
		m_lines.push_back(l2);
	}

}