#pragma once
#include "Mesh.h"
#include <vector>
#include "MathUtils.h"

enum PlacementScheme {CENTER, XZFIELD, XYFIELD, TREE, ORB, TERRAIN, SPIKE};

struct ScaleController //uniform scale
{
	ScaleController() {}
	void Init() {val=0; incr=0.01;}
	void Init(float f) {val=f; incr=0.01;}
	void Update() {val += incr; val = MATH_MIN(val, 1);};
	Vector3D GetVect() {return Vector3D(val, val, val);}
	float GetVal() {return val;}
	float val;
	float incr;
};

struct TranslationController
{
	TranslationController() {}
	void Init(Vector3D v1, Vector3D d) {val=v1; incr=d; useDest = false;}
	void Init(Vector3D v) {val=v; useDest=true; destVal=v; incr=v * 0.01;}
	void Update() {
		val += incr; 
		for(int i=0; useDest && i<3; i++) {
			if(incr[i]>0) val[i] = MATH_MIN(destVal[i], val[i]);
			else val[i] = MATH_MAX(destVal[i], val[i]);
		}
	}
	Vector3D GetVect() {return val;}
	Vector3D val;
	Vector3D incr;
	Vector3D destVal;
	bool useDest;
};

struct FractalScript
{
	void GenerateRandom();
	void ScriptCases();
	struct Line // each line corresponds to a level in the hierarchy.
		// last line of the script must have zero children.
	{
		Line();
		Line(Mesh* m);
		Line(Mesh* m[], int n);
		unsigned int numChildren;
		int placementScheme;
		std::vector<Mesh*> meshes;
		int numMeshes;
		bool useDeform;
		float jitter;

		// Animation info
		bool looping;
		Vector3D rotateIncr;
		Vector3D translateIncr;
	};
	std::vector<Line> m_lines;
};