#pragma once
#include "SubMesh.h"
#include "MeshFlags.h"
#include <vector>
#include <string>

class Mesh
{
public:
	Mesh();
	void Draw(unsigned int numInstances = 1);
	void DrawFires(bool forceMat = false);
	void DrawInstanced(std::vector<Matrix4x4> transforms);
	void DrawCubemap(Texture* t1, Texture* t2, float transition);
	bool LoadObj(std::string fileName, MeshFlags flags);
	std::string GetName() {return m_name;}
	void SetName(std::string s) {m_name = s;}
	float GetExtent(int dim);
	float GetMaxExtent(){return m_maxExtent;}
	Vector3D GetBoundingBox() {return m_upperBound - m_lowerBound;}
	void AttachDeform(Mesh* m);
	void UpdateDeform(float t);
	void SetInstancing(Matrix4x4 m, int index);
	void SetInstancing(std::vector<Matrix4x4> transforms);
	void SetInstancing(std::vector<Matrix4x4> transforms, unsigned int startInd, unsigned int count);
	unsigned int GetNumSubMeshes() {return m_subMeshes.size();}
	SubMesh* GetSubMesh(unsigned int index) {DBG_ASSERT(index<m_subMeshes.size()); return &m_subMeshes[index];}
	void LoadFromBin(std::ifstream& fileStrm, MeshFlags flags);
	void SaveToBin(std::ofstream& fileStrm);
private:
	void CalcBoundingBox();
	std::vector<SubMesh> m_subMeshes;
	std::string m_name;
	bool m_loaded;

	static const int END_OF_MESH;

	// bounding box
	Point3D m_lowerBound;
	Point3D m_upperBound;
	float m_maxExtent;
};

