#include "stdafx.h"
#include "MissileList.h"
#include "AssetManager.h"
#include "transform.h"
#include "SubMesh.h"

void MissileList::Init()
{
#if LOWPOLY
	m_mesh = AssetManager::GetInstance()->GetMesh("missile_lowpoly.obj");
#else
	m_mesh = AssetManager::GetInstance()->GetMesh("missile.obj");
#endif
}

void MissileList::AddMissile(Point3D worldPos, Vector3D dir, Matrix4x4 rot)
{
	if(m_missiles.size() == SubMesh::MAX_INSTANCES) return;
	Missile m;
	m.pos = worldPos;
	m.velocity = dir;
	m.transform = translation(Vector3D(worldPos[0], worldPos[1], worldPos[2]));
	m.rot = rot;
	//m.dir = velocity;
	m_missiles.push_back(m);
}

static float s_farplane;
bool MissileList::outOfBounds (const Missile& m) 
{
	for(int j=0; j<3; j++) {
		if(m.pos[j] < -s_farplane || m.pos[j]>s_farplane) return true;
	}
	return false;
}

void MissileList::Update(Point3D cameraPos, Matrix4x4 cameraRot, float farplane, bool gameLogic)
{
	// Update positions
	int i = 0;
	for(std::list<Missile>::iterator iter = m_missiles.begin(); iter!=m_missiles.end(); iter++)  {
		if(gameLogic) iter->pos = iter->pos + iter->velocity;
		Vector3D transl = cameraPos - iter->pos;
		//iter->transform = cameraRot * translation(transl) * iter->rot;
		iter->transform = translation(transl) * iter->rot;
		m_mesh->SetInstancing(iter->transform, i);
		i++;
	}

	if(!gameLogic) return;

	// Prune missiles that are out of bounds
	s_farplane = farplane;
	m_missiles.remove_if(MissileList::outOfBounds);
}

void MissileList::Draw()
{
	m_mesh->Draw(m_missiles.size());
}

bool MissileList::CollidedWith( Point3D point, Vector3D boxRadius )
{
	if(m_missiles.size()==0) return false;
	for(std::list<Missile>::iterator iter = m_missiles.begin(); iter!=m_missiles.end(); iter++)  {
		if((fabs(iter->pos[0]-point[0])<(boxRadius[0]/2)) && 
			(fabs(iter->pos[1]-point[1])<(boxRadius[1]/2)) && 
			(fabs(iter->pos[2]-point[2])<(boxRadius[2]/2))) {
			m_missiles.erase(iter);
			return true;
		}
	}
	return false;
}