#pragma once
#include "stdafx.h"
#include <sstream>
#include <string>
#include <vector>

#include "PostProcessingGlow.h"
#include "Cg/cgGL.h"

class Model
{
public:
	Model():m_loaded(false) {}
	bool LoadObj(std::string fileName);
	bool LoadMaterials(std::string fileName);
	void SwapYZ();
	void Draw();
private:
	void BuildVBOs();
	struct Vertex
	{
		float position[3];
		float texCoord[2];
		float normal[3];
		float colour[4];
	};
	struct VertexIndices
	{
		unsigned int v;
		unsigned int vt;
		unsigned int vn;
	};
	struct Point2D
	{
		Point2D() {val[0] = -1; val[1] = -1;}
		Point2D(double x, double y) {val[0] = x; val[1] = y;}
		float val[2];
	};
	struct Point3D
	{
		Point3D() {val[0] = -1; val[1] = -1; val[2] = -1;}
		Point3D(double x, double y, double z) {val[0] = x; val[1] = y; val[2]=z;}
		float val[3];
	};
	struct Material
	{
		Point3D ka, kd, ks;
	};

	friend std::istream &operator>>(std::istream &stream, VertexIndices &ob);

	bool m_loaded;
	unsigned int m_vertexVbo;
	unsigned int m_texCoordVbo;
	std::vector<Point3D> m_vertPosList;
	std::vector<Point3D> m_normalList;
	std::vector<Point2D> m_texCoordList;
	std::vector<std::vector<VertexIndices> > m_faces;
	std::vector<Material> m_materials;
	CGprogram m_vertexShader;
	CGprofile m_vertexProfile;
	CGparameter m_proj;
	CGparameter m_view;
	CGparameter m_viewIT;
	CGparameter m_modelViewMatrix;
	CGparameter m_position;
	CGparameter m_color;
	CGparameter m_bend;
	CGparameter m_scale;
	CGparameter m_lightPos;
};

void Model::SwapYZ()
{
	for(int i=0; i<m_vertPosList.size(); i++) {
		double tmp = m_vertPosList[i].val[1];
		m_vertPosList[i].val[1] = m_vertPosList[i].val[2];
		m_vertPosList[i].val[2] = tmp;
	}
}

void Model::BuildVBOs()
{
	// Generate And Bind The Vertex Buffer
	glGenBuffersARB(1, &m_vertexVbo);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, m_vertexVbo);

	//load the data
	//glBufferDataARB( GL_ARRAY_BUFFER_ARB, m_vertexPosns.size()*3*sizeof(double), m_pVertices, GL_STATIC_DRAW_ARB);

	// Generate And Bind The Texture Coordinate Buffer
	glGenBuffersARB(1, &m_texCoordVbo);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, m_texCoordVbo);
	// Load The Data
	//glBufferDataARB( GL_ARRAY_BUFFER_ARB, m_vertices.size()*2*sizeof(double), m_pTexCoords, GL_STATIC_DRAW_ARB);
}

std::istream &operator>>(std::istream &stream, Model::VertexIndices &ob)
{ 
	//read each token in a stream, i.e. a token is v, v//vn, v/vt, or v/vt/vn.
	ob.v = -1;
	ob.vn = -1;
	ob.vt = -1;

	std::string token;
	stream >> token;
	size_t lhslash = token.find('/');
	size_t rhslash = token.rfind('/');
	std::string vStr = token.substr(0, lhslash);
	std::istringstream vStrm(vStr);
	vStrm >> ob.v;
	ob.v--;
	if(lhslash == std::string::npos) {} // format is v
	else if(lhslash + 1 == rhslash) { // format is v//vn
		std::string vnStr = token.substr(rhslash+1, token.size());
		std::istringstream vnStrm(vnStr);
		vStrm >> ob.vn;
		ob.vn--; // use 0-indexing
	}
	else if(lhslash == rhslash) { // v/vt
		std::string vtStr = token.substr(rhslash+1, token.size());
		std::istringstream vtStrm(vtStr);
		vtStrm >> ob.vt;
		ob.vt--;
	}
	else { // v/vt/vn.
		std::string vtStr = token.substr(lhslash+1, rhslash);
		std::istringstream vtStrm(vtStr);
		vtStrm >> ob.vt;
		ob.vt--;
		std::string vnStr = token.substr(rhslash+1, token.size());
		std::istringstream vnStrm(vnStr);
		vnStrm >> ob.vn;
		ob.vn--;
	}
	return stream;
}

bool Model::LoadObj(std::string fileName)
{
	std::ifstream fileStream(fileName.c_str());
	if(!fileStream.good()) return false;

	while (fileStream.good())
	{
		std::string tag;
		fileStream >> tag;

		if(!tag.compare("f")) {
			std::string indexList;
			getline(fileStream, indexList);
			std::istringstream indexListStrm(indexList);
			std::vector<VertexIndices> face;
			while(1) {
				VertexIndices indices;
				indexListStrm >> indices;
				if(!indexListStrm.good()) break;
				face.push_back(indices);
			}
			m_faces.push_back(face);
		}
		else if(!tag.compare("mtllib")) {
			std::string workingDir;
			size_t ind1 = fileName.rfind('/');
			size_t ind2 = fileName.rfind('\\');
			if(ind1 < fileName.size()-1) workingDir += fileName.substr(0, ind1+1);
			else if(ind2 < fileName.size()-1) workingDir += fileName.substr(0, ind2+1);
			std::string mtllibList;
			getline(fileStream, mtllibList);
			std::istringstream mtllibListStrm(mtllibList);
			while(1) {
				std::string libName;
				mtllibListStrm >> libName;
				if(!mtllibListStrm.good()) break;
				LoadMaterials(workingDir + libName);
			}
		}
		else if(!tag.compare("v")) {
			double v[3];
			fileStream >> v[0] >> v[1] >> v[2];
			m_vertPosList.push_back(Point3D(v[0], v[1], v[2]));
		}
		else if(!tag.compare("vn")) {
			double vn[3];
			fileStream >> vn[0] >> vn[1] >> vn[2];
			m_normalList.push_back(Point3D(vn[0], vn[1], vn[2]));
		}
		else if(!tag.compare("vt")) {
			double vt[2];
			fileStream >> vt[0] >> vt[1];
			m_texCoordList.push_back(Point2D(vt[0], vt[1]));
		}
	}

	CGcontext cgContext;

	cgContext = cgCreateContext();
	DBG_ASSERT(cgContext != NULL);
	m_vertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	DBG_ASSERT(m_vertexProfile != CG_PROFILE_UNKNOWN);
	cgGLSetOptimalOptions(m_vertexProfile);
	m_vertexShader = cgCreateProgramFromFile(cgContext, CG_SOURCE, "data/Shaders/Deform.cg", m_vertexProfile, "main", 0);
	DBG_ASSERT(m_vertexShader);
	cgGLLoadProgram(m_vertexShader);

	m_position = cgGetNamedParameter(m_vertexShader, "IN.position");
	m_color = cgGetNamedParameter(m_vertexShader, "IN.color");
	m_bend = cgGetNamedParameter(m_vertexShader, "IN.bend");
	m_scale = cgGetNamedParameter(m_vertexShader, "IN.scale");
	m_modelViewMatrix = cgGetNamedParameter(m_vertexShader, "ModelViewProj");
	m_view = cgGetNamedParameter(m_vertexShader, "View");
	m_viewIT = cgGetNamedParameter(m_vertexShader, "ViewIT");
	m_lightPos = cgGetNamedParameter(m_vertexShader, "LightPos");
	
	m_loaded = true;
	return true;
}

bool Model::LoadMaterials(std::string fileName)
{
	std::ifstream fileStream(fileName.c_str());
	if(!fileStream.good()) return false;

	Material material;

	while (fileStream.good())
	{
		std::string tag;
		fileStream >> tag;

		if(!tag.compare("newmtl")) {
			std::string name;
			fileStream >> name;
		}
		else if(!tag.compare("Ka")) {
			fileStream >> material.ka.val[0] >> material.ka.val[1] >> material.ka.val[2];			
		}
	}

	m_materials.push_back(material);
	return true;
}

void Model::Draw()
{
	if(!m_loaded) return;

	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_DEPTH_TEST);

	cgGLSetStateMatrixParameter(m_modelViewMatrix, CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);

	cgGLSetStateMatrixParameter(m_proj,CG_GL_PROJECTION_MATRIX,CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(m_view, CG_GL_MODELVIEW_MATRIX,CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(m_viewIT, CG_GL_MODELVIEW_MATRIX,CG_GL_MATRIX_INVERSE_TRANSPOSE);
	cgGLSetParameter3f(m_lightPos, 0, 0, 0);

	// Bind shader
	//cgGLEnableProfile(m_vertexProfile);
	//cgGLBindProgram(m_vertexShader);
	cgGLSetParameter4f(m_color, 1.0f, 0.0f, 0.0f, 1.0f);
	cgGLSetParameter1f(m_bend, 0.5);
	cgGLSetParameter3f(m_scale, 1.0, 1.0, 1.0);

	for(int i=0; i<m_faces.size(); i++) {
		std::vector<VertexIndices>& indices = m_faces[i];
		if(indices.size()==4) {
			glBegin(GL_QUADS);
			for(int j=0; j<4; j++) {
				glNormal3d(m_normalList[indices[j].vn].val[0], m_normalList[indices[j].vn].val[1], m_normalList[indices[j].vn].val[2]);
				glTexCoord2d(m_texCoordList[indices[j].vt].val[0], m_texCoordList[indices[j].vt].val[1]);
				glVertex3d(m_vertPosList[indices[j].v].val[0], m_vertPosList[indices[j].v].val[1], m_vertPosList[indices[j].v].val[2]);
			}
			glEnd();
		}
		else if(indices.size()==3) {
			glBegin(GL_TRIANGLES);
			for(int j=0; j<3; j++) {
				glNormal3d(m_normalList[indices[j].vn].val[0], m_normalList[indices[j].vn].val[1], m_normalList[indices[j].vn].val[2]);
				glTexCoord2d(m_texCoordList[indices[j].vt].val[0], m_texCoordList[indices[j].vt].val[1]);
				glVertex3d(m_vertPosList[indices[j].v].val[0], m_vertPosList[indices[j].v].val[1], m_vertPosList[indices[j].v].val[2]);
			}
			glEnd();

		}
	}

	glClearColor(0,0,0,0);
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glDisable(GL_FRAGMENT_PROGRAM_ARB);
	glDisable(GL_VERTEX_PROGRAM_ARB);

	for(int i=0; i<m_faces.size(); i++) {
		std::vector<VertexIndices>& indices = m_faces[i];
		if(indices.size()==4) {
			glBegin(GL_QUADS);
			for(int j=0; j<4; j++) {
				glNormal3d(m_normalList[indices[j].vn].val[0], m_normalList[indices[j].vn].val[1], m_normalList[indices[j].vn].val[2]);
				glTexCoord2d(m_texCoordList[indices[j].vt].val[0], m_texCoordList[indices[j].vt].val[1]);
				glVertex3d(m_vertPosList[indices[j].v].val[0], m_vertPosList[indices[j].v].val[1], m_vertPosList[indices[j].v].val[2]);
			}
			glEnd();
		}
		else if(indices.size()==3) {
			glBegin(GL_TRIANGLES);
			for(int j=0; j<3; j++) {
				glNormal3d(m_normalList[indices[j].vn].val[0], m_normalList[indices[j].vn].val[1], m_normalList[indices[j].vn].val[2]);
				glTexCoord2d(m_texCoordList[indices[j].vt].val[0], m_texCoordList[indices[j].vt].val[1]);
				glVertex3d(m_vertPosList[indices[j].v].val[0], m_vertPosList[indices[j].v].val[1], m_vertPosList[indices[j].v].val[2]);
			}
			glEnd();

		}
	}

	//cgGLDisableProfile(m_vertexProfile);
}
