#include "stdafx.h"
#include "ShaderProgram.h"

void GetProgramDebugInfo(GLuint obj)
{
	int infologLength = 0;
	int charsWritten  = 0;
	char infoLog[1024];

	glGetProgramInfoLog(obj, 1024, &charsWritten, infoLog);
}

void ShaderProgram::Compile(Shader& vs, Shader& fs, bool inst, bool dfm, bool post)
{
	m_id = glCreateProgram();

	glAttachShader(m_id, vs.GetId());
	glAttachShader(m_id, fs.GetId());
	glLinkProgram(m_id);
	int result;
	glGetProgramiv(m_id, GL_LINK_STATUS, &result);
	if(result!=GL_TRUE) {
		GetProgramDebugInfo(m_id);
		DBG_ASSERT(0);
	}
	int numAttribs = 0;
	int maxAttribLen = 0;
	glGetProgramiv(m_id, GL_ACTIVE_ATTRIBUTES, &numAttribs);
	glGetProgramiv(m_id, GL_ACTIVE_ATTRIBUTE_MAX_LENGTH, &maxAttribLen);
	char* attribNameBuf = new char[maxAttribLen];
	for(int i=0; i<numAttribs; i++) {
		Variable v;
		int len=0;
		glGetActiveAttrib(m_id, i, maxAttribLen, &len, &v.size, &v.type, attribNameBuf);
		v.name = attribNameBuf;
		v.location = glGetAttribLocation(m_id, attribNameBuf);
		m_vars[v.name] = v;
	}
	delete[] attribNameBuf;

	int numUniforms = 0;
	int maxUniformLen = 0;
	glGetProgramiv(m_id, GL_ACTIVE_UNIFORMS, &numUniforms);
	glGetProgramiv(m_id, GL_ACTIVE_UNIFORM_MAX_LENGTH, &maxUniformLen);
	char* uniformNameBuf = new char[maxUniformLen];
	for(int i=0; i<numUniforms; i++) {
		int len=0;
		Variable v;
		glGetActiveUniform(m_id, i, maxUniformLen, &len, &v.size, &v.type, uniformNameBuf);
		v.name = uniformNameBuf;
		v.location = glGetUniformLocation(m_id, uniformNameBuf);
		m_vars[v.name] = v;

	}
	delete[] uniformNameBuf;

	m_instancing = inst;
	m_deform = dfm;
	m_postProc = post;

}

unsigned int ShaderProgram::GetVarId( std::string name )
{
	if(m_vars.find(name) == m_vars.end()) {
		//DBG_ASSERT(false);
		return 0xFFFFFFFF;
	}
	return m_vars[name].location;
}