#include "stdafx.h"
#include "Slider.h"
#include "AssetManager.h"
#include "Glyphs.h"
#include "MathUtils.h"
#include <sstream>

Slider* Slider::s_picked_up = 0;

void Slider::DrawQuad(float x, float y, float width, float height)
{
	glBegin(GL_QUADS);
	glVertex2d(x, y);
	glVertex2d(x+width, y);
	glVertex2d(x+width, y+height);
	glVertex2d(x, y+height);
	glEnd();
}

void Slider::DrawMarker(float x, float y)
{
	glBegin(GL_TRIANGLES);
	glVertex2d(x-MARKSIZE, y+(2*MARKSIZE));
	glVertex2d(x+MARKSIZE, y+(2*MARKSIZE));
	glVertex2d(x, y);
	glEnd();
}

void Slider::Draw(unsigned int index, float maxx, float maxy)
{
	// Draw bar
	DrawQuad(SLIDER_LMARGIN + SLIDER_SPACE_LBL_BAR, maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN, SLIDER_WIDTH, 2);

	// Draw indicator
	if(m_type == PERCENT) {
		DrawMarker(SLIDER_LMARGIN + SLIDER_SPACE_LBL_BAR + (SLIDER_WIDTH*(*m_percent)), maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN);
	}
	else if(m_type == DISCRETE) {
		float effPct = (float)(*m_discrete)/(float)(m_hi - m_lo);
		DrawMarker(SLIDER_LMARGIN + SLIDER_SPACE_LBL_BAR + (SLIDER_WIDTH*(effPct)), maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN);
	}

	// Draw label
	Glyphs::ShowText(m_label, SLIDER_LMARGIN, maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN, maxx, maxy, true);

	// Draw number text
	std::stringstream num;
	if(m_type == PERCENT) {
		num.precision(2);
		num.setf(std::ios_base::fixed);
		num << (*m_percent);
	}
	else if(m_type == DISCRETE) {
		num << (*m_discrete);
	}
	Glyphs::ShowText(num.str(), SLIDER_LMARGIN+ SLIDER_SPACE_LBL_BAR+SLIDER_WIDTH, maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN, maxx, maxy);
}

bool Slider::IsOnHotspot(unsigned int index, float maxx, float maxy, Point2D pos)
{
	const float CLICK_RADIUS = MARKSIZE*MARKSIZE*5;
	int markerX = 0;
	int markerY = 0;
	
	if(m_type == PERCENT) {
		markerX = SLIDER_LMARGIN + SLIDER_SPACE_LBL_BAR + (SLIDER_WIDTH*(*m_percent));
		markerY = maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN;
	}
	else if(m_type == DISCRETE) {
		float effPct = (float)(*m_discrete)/(float)(m_hi - m_lo);
		markerX = SLIDER_LMARGIN + SLIDER_SPACE_LBL_BAR + (SLIDER_WIDTH*effPct);
		markerY = maxy-(SLIDER_VSPACING*index)-SLIDER_TMARGIN;
	}
	if(((pos[0]-markerX)*(pos[0]-markerX))+((pos[1]-markerY)*(pos[1]-markerY)) < CLICK_RADIUS) {
		return true;
	}
	return false;
}

void Slider::PickUp()
{
	s_picked_up = this;
	DBG_ASSERT(m_type == PERCENT || m_type == DISCRETE);
}

void Slider::PutDown() 
{
	DBG_ASSERT(s_picked_up->m_type == PERCENT || s_picked_up->m_type == DISCRETE);
	s_picked_up = NULL;
}
Slider* Slider::GetPickedUp()
{
	if(s_picked_up == NULL) return NULL;
	DBG_ASSERT(s_picked_up->m_type == PERCENT || s_picked_up->m_type == DISCRETE);
	return s_picked_up;
}

void Slider::Adjust(float dx)
{
	DBG_ASSERT(s_picked_up);
	if(m_type == PERCENT) {
		(*m_percent) += dx/(float)SLIDER_WIDTH;
		(*m_percent) = MATH_MAX((*m_percent), 0);
		(*m_percent) = MATH_MIN((*m_percent), 1);
	}
	else if(m_type == DISCRETE) {
		dragDelta += dx;
		if(dragDelta > SLIDER_WIDTH/(float)(m_hi - m_lo) && (*m_discrete)<m_hi) { (*m_discrete)++; dragDelta = 0;}
		else if(dragDelta < -SLIDER_WIDTH/(float)(m_hi - m_lo) && (*m_discrete)>m_lo) { (*m_discrete)--; dragDelta = 0;}
	}
	
}