#pragma once

#include <assert.h>
#include <al.h>
#include <alc.h>
#include <vorbis/vorbisfile.h>
#include <vector>
#include <fstream>

class SongPlayer
{
public:
	void InitDevice()
	{
		// Open default device
		ALCdevice* alDevice = alcOpenDevice("");
		ALenum err;
		DBG_ASSERT(!alGetError());

		ALCcontext* context=alcCreateContext(alDevice,NULL);
		alcMakeContextCurrent(context);

		alGenBuffers(1, &alBuffNum);
		DBG_ASSERT(!alGetError());

		alGenSources(1,	&alSrcNum);
		DBG_ASSERT(!alGetError());

		alListener3f(AL_POSITION, 0.0f, 0.0f, 0.0f);
		alSource3f(alSrcNum, AL_POSITION, 0.0f, 0.0f, 0.0f);
	}
	void LoadFile(std::string fileName)
	{
		int endian = 0;             // 0 for Little-Endian, 1 for Big-Endian
		int bitStream;
		long bytes;
		char array[BUFFER_SIZE];    // Local fixed size array
		FILE *f;

		// Open for binary reading
		f = fopen(fileName.c_str(), "rb");
		DBG_ASSERT(f);

		ov_open(f, &oggFile, NULL, 0);
		// Get some information about the OGG file
		pInfo = ov_info(&oggFile, -1);

		// Check the number of channels... always use 16-bit samples
		if (pInfo->channels == 1)
			format = AL_FORMAT_MONO16;
		else
			format = AL_FORMAT_STEREO16;
		// end if

		// The frequency of the sampling rate
		freq = pInfo->rate;
		do {
			// Read up to a buffer's worth of decoded sound data
			bytes = ov_read(&oggFile, array, BUFFER_SIZE, endian, 2, 1, &bitStream);
			// Append to end of buffer
			buffer.insert(buffer.end(), array, array + bytes);
		} while (bytes > 0);
		ov_clear(&oggFile);

		alBufferData(alBuffNum, format,  &buffer[0], buffer.size(), freq);

		// Attach source to buffer
		alSourcei(alSrcNum, AL_BUFFER, alBuffNum);
		DBG_ASSERT(!alGetError());

	}

	void Play()
	{
		alSourcePlay(alSrcNum);
		DBG_ASSERT(!alGetError());
	}
	void Update()
	{
		alGetSourcei(alSrcNum, AL_SOURCE_STATE, &m_state);
	}
	bool IsDone(){return m_state==AL_STOPPED;}
	void Stop()
	{
	}
	void Shutdown()
	{
		alDeleteBuffers(1, &alBuffNum);
		alDeleteSources(1, &alSrcNum);
	}
private:
	static const int BUFFER_SIZE = 32768;
	ALuint alBuffNum;
	ALuint alSrcNum;
	std::vector<char> buffer;
	ALenum format;
	ALsizei freq;
	vorbis_info *pInfo;
	OggVorbis_File oggFile;
	ALint m_state;
};
