#pragma once
#include "stdafx.h"
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include "MeshFlags.h"
#include "Texture.h"
#include "Material.h"

struct Vertex
{
	Point3D position;
	Point2D texCoord;
	Vector3D normal;
};
struct VertexIndices
{
	VertexIndices(): useVt(true), useVn(true) {}
	unsigned int v;
	unsigned int vt;
	unsigned int vn;
	bool useVt;
	bool useVn;
};
std::istream &operator>>(std::istream &stream, VertexIndices &ob);

class SubMesh
{
public:
	SubMesh():m_loaded(false){}
	bool LoadObj(std::string fileName, MeshFlags flags);
	void SaveToBin(std::ofstream& fileStrm);
	void LoadFromBin(std::ifstream& fileStrm, MeshFlags flags);
	std::string GetName() {return m_name;}
	void SetName(std::string s) {m_name = s;}
	void Translate(Vector3D amount);
	void SwapYZ();
	void Draw(int numInstances=1);
	void DrawFire(bool forceMat = false);
	void DrawInstanced(std::vector<Matrix4x4> transforms);
	void DrawCubemap(Texture* t1, Texture* t2, float transition);
	void ApplyMaterial();
	void BuildVBOs();
	bool IsDeform() {return false;}
	bool AttachDeform(SubMesh* m_);
	void UpdateDeform(double percent);
	void SetInstancing(Matrix4x4 m, int index);
	void SetInstancing(std::vector<Matrix4x4> transforms);
	void SetInstancing(std::vector<Matrix4x4> transforms, unsigned int startInd, unsigned int count);
	float GetExtent(int dim);
	float GetMaxExtent(){return m_maxExtent;}
	Point3D GetLowerBound() {return m_lowerBound;}
	Point3D GetUpperBound() {return m_upperBound;}
	unsigned int GetNumIndices() {return m_numFaces*3;}
	unsigned int GetNumVertices() {return m_numPositions;}
	unsigned int GetNumNormals() {return m_numPositions;}
	//Point3D GetVertex(unsigned int i) {return m_vertPosList[i];}
	//unsigned int GetIndex(unsigned int i) {return m_indexList[i];}
	//Vector3D GetNormal(unsigned int i) {return m_normalList[i];}
	float* GetPositionBuffer() {return m_positionBuf;}
	float* GetNormalBuffer() {return m_normalBuf;}
	void SetMaterial(Material m);
	Material GetMaterial() {return m_material;}
	void AddTriangle(Vertex v1, Vertex v2, Vertex v3);
	void SetTitle(std::string title) {m_title = title;}
	void CalcBoundingBox();
	static const int MAX_INSTANCES = 50;
protected:
	void SetInstanceDrawParams();
	void SetDeformDrawParams();
	void BuildVerticesFromRaw();

	static const int END_OF_SUBMESH;

	class InstanceData
	{
	public:
		InstanceData();
		void SetTrans(int i, Matrix4x4 trans);
		float* GetBuffer() {return (float*)m_instanceTransBuf;}
	private:
		float m_instanceTransBuf[SubMesh::MAX_INSTANCES][4][4];
	} m_instanceData;

	bool m_loaded;
	MeshFlags m_flags;

	unsigned int m_vertexVbo;
	unsigned int m_vertexIndexBuf;
	unsigned int m_texCoordVbo;
	unsigned int m_normalVbo;
	unsigned int m_instanceIdBuf;
	unsigned int m_numFaces; // because an indexList vector isn't always there
	unsigned int m_numPositions;
	float* m_positionBuf;
	float* m_normalBuf;

	Material m_material;
	std::string m_name;
	std::string m_materialName;
	std::string m_title;

	std::vector<Point3D> m_vertPosList;
	std::vector<Vector3D> m_normalList;
	std::vector<Point2D> m_texCoordList;
	std::vector<Matrix4x4> m_instanceTransformList;
	std::vector<unsigned int> m_indexList;
	Point3D m_lowerBound;
	Point3D m_upperBound;
	float m_maxExtent;

	std::vector<std::vector<VertexIndices> > m_faces;
	std::vector<Material> m_materials;

	static Texture m_buildingSegTex;
	static Point3D m_ambient;

	struct DeformInfo
	{
		unsigned int m_deformPosBuf;
		unsigned int m_deformNormBuf;
		unsigned int m_deformSpeedBuf;
		std::vector<Point3D> m_deformPosList;
		double m_percent;
	};
	DeformInfo m_deformInfo;
	Matrix4x4 m_transform;

};
