#include "stdafx.h"
#include "Texture.h"
#include "cairo/cairo.h"
#include "AssetManager.h"

bool Texture::Load(std::string path, std::string fileName)
{
	cairo_surface_t *surface;
	cairo_t *cr;
	surface = cairo_image_surface_create_from_png((path+fileName).c_str());
	cr = cairo_create(surface);

	// check format
	cairo_format_t surfFmt = cairo_image_surface_get_format(surface);
	// cairo surface format is _RGB.. need to convert to RGBA

	int width = cairo_image_surface_get_width(surface);
	int height = cairo_image_surface_get_height(surface);
	m_width = width;
	m_height = height;

	// allocate gl texture
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &m_id);
	glBindTexture(GL_TEXTURE_2D, m_id);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND); //allow transparency
	glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, cairo_image_surface_get_data(surface));

	// build mipmaps
	int ret = gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, width, height, GL_RGBA, GL_UNSIGNED_BYTE, cairo_image_surface_get_data(surface));
	DBG_ASSERT(ret==0);

	// free cairo surface
	cairo_destroy (cr);
	cairo_surface_destroy (surface);

	m_name = fileName;
	m_loaded = true;
	return true;
}

bool Texture::Create(unsigned int width, unsigned int height)
{
	m_width = width;
	m_height = height;

	glGenTextures(1, &m_id);
	DBG_ASSERT(m_id>0);
	glBindTexture(GL_TEXTURE_2D, m_id);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
	return true;
}

bool Texture::CreateDepth(unsigned int width, unsigned int height)
{
	m_width = width;
	m_height = height;

	glGenTextures(1, &m_id);
	DBG_ASSERT(m_id>0);
	glBindTexture(GL_TEXTURE_2D, m_id);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, width, height, 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);

	m_loaded = true;

	return true;

}

bool Texture::CreateUnfilled()
{
	glGenTextures(1, &m_id);
	return true;
}

void Texture::Free()
{
	glDeleteTextures(1, &m_id);
	m_loaded = false;
}

bool Texture::LoadCubemap(std::string prefix, std::string extension )
{
	const std::string suffixes[] = {"_positive_x", "_positive_y", "_positive_z", "_negative_x", "_negative_y", "_negative_z"};
	const unsigned int targets[] = {GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT, GL_TEXTURE_CUBE_MAP_NEGATIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z};

	glGenTextures(1, &m_id);
	glEnable(GL_TEXTURE_CUBE_MAP);
	glBindTexture(GL_TEXTURE_CUBE_MAP, m_id);

	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);

	for(int i=0; i<6; i++) {
		cairo_surface_t *surface = 0;
		cairo_t *cr = 0;
		std::string fileName = prefix + suffixes[i] + extension;
		surface = cairo_image_surface_create_from_png((AssetManager::GetInstance()->GetDataPath() + fileName).c_str());
		DBG_ASSERT(surface);
		cr = cairo_create(surface);
		DBG_ASSERT(cr);
		int width = cairo_image_surface_get_width(surface);
		int height = cairo_image_surface_get_height(surface);
		m_width = width;
		m_height = height;

		// allocate gl texture
		glTexImage2D(targets[i], 0, 4, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, cairo_image_surface_get_data(surface));

		cairo_destroy (cr);
		cairo_surface_destroy (surface);
	}

	m_name = prefix;
	return true;
}