#include "stdafx.h"

//Scenes
#include "MissileList.h"
#include "EnemyList.h"
#include "Player.h"
#include "Explosion.h"

#include "SongPlayer.h"
#include "AssetManager.h"
#include "GlobalSettings.h"
#include "MathUtils.h"
#include "algebra.h"
#include "trackball.h"
#include "Slider.h"
#include "Random.h"
#include "UI.h"
#include "Mesh.h"
#include "Glyphs.h"
#include <vector>
#include <fstream>
#include <string>
#include <time.h>
#include <list>

#define ASSERT(x) assert(x)

SongPlayer gSongPlayer;

AssetManager assetManager;
GlobalSettings g_globalSettings;

struct Light
{
	Point3D worldPos;
	Point3D col;
};

// Items in the scene
Player s_player;
MissileList s_missileList;
EnemyList s_enemyList;
unsigned int s_score;
float s_frameCounter = 0;
std::list<Explosion> s_explosions;
std::vector<Light> s_lights;
std::vector<Slider> s_sliders;
bool s_show_ui = false;
bool s_paused = false;
float ticksTilSpawn = 0;
Matrix4x4 s_env_rot;

// parameters
float s_spawnfreq = 0.5;
unsigned int s_showSmoke = 0;
unsigned int s_playerMeshNum = 0;

int currentSceneIndex = 0;

Point3D s_cameraPos;
Matrix4x4 s_cameraRot;

bool IsExtensionSupported( char* szTargetExtension );

struct UIState
{
	UIState(): b1(false), b2(false), b3(false), glReady(false) {}
	int lastMouseX;
	int lastMouseY;
	double screenWidth, screenHeight;
	double trackballRadius;
	bool b1, b2, b3;
	bool glReady;
} uiState;

void OnAppInit(AppInitArgs args)
{
	DBG_ASSERT(IsExtensionSupported("GL_ARB_vertex_buffer_object"));
	DBG_ASSERT(IsExtensionSupported("GL_ARB_vertex_shader"));
	DBG_ASSERT(IsExtensionSupported("GL_ARB_fragment_shader"));
	DBG_ASSERT(IsExtensionSupported("GL_EXT_framebuffer_object"));	 

	g_globalSettings.density = 0.5;
	
	glewInit();
	AssetManager::GetInstance()->LoadAll();
	Glyphs::CreateTexture();

	// init scene objects
	s_missileList.Init();
	s_enemyList.Init();
	Light l = {Point3D(0, 0, 0), Point3D(1, 0, 0)};
	s_lights.push_back(l);

	s_cameraPos = Point3D(0, 0, 0);
	s_player.m_pos = Point3D(0, 5, 50);
	//s_enemyList.AddEnemy(Point3D(0, 0, 100));

	srand(time(0));
	
	AssetManager::GetInstance()->AddParameter("Show fires", &s_showSmoke, 0, 1);
	AssetManager::GetInstance()->AddParameter("Enemy spawn rate", &s_spawnfreq);	
	AssetManager::GetInstance()->AddParameter("Player model", &s_playerMeshNum, 0, 1);
	

	uiState.glReady = true;
}

void OnMouseMove(MouseMoveArgs args)
{
	//Scene* currentScene = sceneList[currentSceneIndex].scene;
	if(Slider::GetPickedUp()) {
		Slider::GetPickedUp()->Adjust(args.x - uiState.lastMouseX);
	}
	else if(uiState.b1 && !s_show_ui) {
		const double dampen = 0.01;
		double dx = args.x - uiState.lastMouseX;
		double dy = args.y - uiState.lastMouseY;
		dx = dampen * dx;
		dy = dampen * dy;
		//currentScene->Translate(Vector3D(dx, -dy, 0));
	}
	else if(uiState.b2 && !s_show_ui) {
		const double dampen = 0.01;
		double dy = args.y - uiState.lastMouseY;
		dy = dampen * dy;
		//currentScene->Translate(Vector3D(0, 0, dy));
	}
	else if(uiState.b3 && !s_show_ui) {
		double rotVec[3];
		double oldx = uiState.lastMouseX - (uiState.screenWidth/2);
		double oldy = uiState.lastMouseY - (uiState.screenHeight/2);
		double newx = args.x - (uiState.screenWidth/2);
		double newy = args.y - (uiState.screenHeight/2);
		vCalcRotVec(oldx, oldy, newx, newy, 100, &rotVec[0], &rotVec[1], &rotVec[2]);
		Matrix4x4 newRot;
		vAxisRotMatrix(rotVec[0], rotVec[1], rotVec[2], &newRot);
		Vector3D angleInc(rotVec[0], rotVec[1], rotVec[2]);
		angleInc = angleInc * (180.0 / 3.14159);
		//s_cameraDir += angleInc;
		//currentScene->RotateL(newRot);
		s_cameraRot = newRot * s_cameraRot;
	}
	uiState.lastMouseX = args.x;
	uiState.lastMouseY = args.y;
}

void DrawPlayer()
{
	glPushMatrix();
	Vector3D trans = s_cameraPos - s_player.m_pos;
	//s_cameraRot.mult_gl(); //whole scene rotation
	glTranslated(trans[0],trans[1],trans[2]); // whole scene translation
	s_player.m_rotTrans.mult_gl();
	if(s_playerMeshNum == 1) {
		AssetManager::GetInstance()->GetMesh("spaceship3.obj")->Draw();
	}
	else if(s_playerMeshNum == 0) {
		AssetManager::GetInstance()->GetMesh("spaceship.obj")->Draw();
	}
	glPopMatrix();
}

void RenderScene(float width, float height)
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50, width/height , 1, 2000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);

	//Draw skybox at (0, 0, 0), rotated according to camera
	glPushMatrix();
	s_cameraRot.mult_gl();
	AssetManager::GetInstance()->GetEnvironmentMap()->Draw();
	glUniform1f(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("explosionFlag"), 0);

	// Draw all the non-instanced stuff
	AssetManager::GetInstance()->UseProgram(AssetManager::GetInstance()->GetNonInstanceProg());
	DrawPlayer(); // player

	// Draw instanced groups
	AssetManager::GetInstance()->UseProgram(AssetManager::GetInstance()->GetInstanceProg());
	s_missileList.Draw();
	s_enemyList.Draw();
	
	// Draw explosions
	for(std::list<Explosion>::iterator iter = s_explosions.begin(); iter!= s_explosions.end(); iter++) {
		glPushMatrix();
		Vector3D trans = s_cameraPos - iter->GetPos();
		//s_cameraRot.mult_gl(); //whole scene rotation
		glTranslated(trans[0],trans[1],trans[2]); // whole scene translation
		iter->Draw();
		glPopMatrix();

	}
	
	glPopMatrix();
}

void DrawFires(float scrWidth, float scrHeight)
{
	//Draw fires on player
	glPushMatrix();
	Vector3D trans = s_cameraPos - s_player.m_pos;
	s_cameraRot.mult_gl(); //whole scene rotation
	glTranslated(trans[0],trans[1],trans[2]); // whole scene translation
	s_player.m_rotTrans.mult_gl();
	if(s_playerMeshNum == 1) {
		AssetManager::GetInstance()->GetMesh("spaceship3.obj")->DrawFires();
	}
	else if(s_playerMeshNum == 0) {
		AssetManager::GetInstance()->GetMesh("spaceship.obj")->DrawFires();
	}
	glPopMatrix();
}

void DrawHUD(float width, float height)
{
	const float SCALE = 0.8;
	Texture* hudTex = AssetManager::GetInstance()->GetTexture("hud.png");
	glClear(GL_DEPTH_BUFFER_BIT);
	glActiveTexture(GL_TEXTURE0 + 0);
	glBindTexture(GL_TEXTURE_2D, hudTex->GetId());
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
	glBegin(GL_QUADS);
	glTexCoord2d(0, 1);
	glVertex2d(0, 0);
	glTexCoord2d(0, 0);
	glVertex2d(0, hudTex->GetHeight()*SCALE);
	glTexCoord2d(1, 0);
	glVertex2d(hudTex->GetWidth()*SCALE, hudTex->GetHeight()*SCALE);
	glTexCoord2d(1, 1);
	glVertex2d(hudTex->GetWidth()*SCALE, 0);
	glEnd();

	if(s_paused)
	{
		Glyphs::ShowText("Paused", width/2, height/2, width, height);
	}
	if(s_show_ui) {
		for(int i=0; i<AssetManager::GetInstance()->GetNumSliders(); i++) {
			AssetManager::GetInstance()->GetSlider(i).Draw(i, width, height);
		}
	}
	
	// Show score
	int scoreX = hudTex->GetWidth()*SCALE*.7;
	int scoreY = hudTex->GetHeight()*SCALE*.22;
	std::stringstream strStrm;
	strStrm << s_score;
	Glyphs::ShowText(strStrm.str(), scoreX, scoreY, width, height);

	glDisable(GL_BLEND);
}

void DrawPostProcessing(float width, float height)
{
	glDisable(GL_DEPTH_TEST);
	//glClear(GL_DEPTH_BUFFER_BIT);
	glClear(GL_COLOR_BUFFER_BIT);

	AssetManager::GetInstance()->UseProgram(0);
	glEnable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0 + 0);
	glBindTexture(GL_TEXTURE_2D, AssetManager::GetInstance()->GetOffscreenTexture(width, height)->GetId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	glDisable(GL_TEXTURE_GEN_S); 
	glDisable(GL_TEXTURE_GEN_T); 
	glDisable(GL_TEXTURE_GEN_R);
	glDisable(GL_TEXTURE_CUBE_MAP);

	//Draw what's been rendered offscreen
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, width, 0, height);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glBegin(GL_QUADS);
	glTexCoord2d(0, 0);
	glVertex2d(0, 0);
	glTexCoord2d(0, 1);
	glVertex2d(0, height);
	glTexCoord2d(1, 1);
	glVertex2d(width, height);
	glTexCoord2d(1, 0);
	glVertex2d(width, 0);
	glEnd();

	//Draw fireball
	glEnable(GL_DEPTH_TEST);
	glClear(GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50, width/height , 0.1, 2000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	AssetManager::GetInstance()->UseProgram(AssetManager::GetInstance()->GetPostProcessingProg());
	// offscreen buf texture
	glActiveTexture(GL_TEXTURE0 + 0);
	glBindTexture(GL_TEXTURE_2D, AssetManager::GetInstance()->GetOffscreenTexture(width, height)->GetId());
	glEnable(GL_TEXTURE_2D);
	glUniform1i(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("offscrTex"), 0);

	// other reference texture
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, AssetManager::GetInstance()->GetTexture("fire.png")->GetId());
	glEnable(GL_TEXTURE_2D);
	glUniform1i(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("auxTex"), 1);

	// depth buffer
	glActiveTexture(GL_TEXTURE0 + 2);
	glBindTexture(GL_TEXTURE_2D, AssetManager::GetInstance()->GetDepthBufferTex(width, height)->GetId());
	glEnable(GL_TEXTURE_2D);
	glUniform1i(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("depthBuf"), 2);

	glUniform1f(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("scrWidth"), width);
	glUniform1f(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("scrHeight"), height);
	glUniform1f(AssetManager::GetInstance()->GetCurrentProgram()->GetVarId("time"), s_frameCounter*.35);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_REPLACE);
	if(s_showSmoke) DrawFires(width, height);
	// unbind fire textures
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, 0);
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, 0);
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_DEPTH_TEST);

	// Draw HUD and 2d elements
	AssetManager::GetInstance()->UseProgram(0);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, width, 0, height);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	DrawHUD(width, height);
}

void OnWindowPaint(WindowPaintArgs args)
{
	if(!uiState.glReady) return;
	uiState.screenWidth = args.width;
	uiState.screenHeight = args.height;
	uiState.trackballRadius = MATH_MIN(args.width, args.height)*.9;
	
	// Draw to offscreen
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, AssetManager::GetInstance()->GetFbo()); //bind fbo
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, AssetManager::GetInstance()->GetOffscreenTexture(args.width, args.height)->GetId(), 0);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_TEXTURE_2D, AssetManager::GetInstance()->GetDepthBufferTex(args.width, args.height)->GetId(), 0);
	glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);

	GLenum status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
	if (status != GL_FRAMEBUFFER_COMPLETE_EXT) return;

	glClearDepth(1.0f);
	// Render the scene
	RenderScene(args.width, args.height);

	//Unbind offscreen fbo
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	glDrawBuffer(GL_BACK);
	glReadBuffer(GL_BACK);

	DrawPostProcessing(args.width, args.height);
}

void OnWindowResize(WindowResizeArgs args)
{

}

void OnMouseBtnDown(MouseBtnDownArgs args)
{
	uiState.lastMouseX = args.x;
	uiState.lastMouseY = args.y;
	uiState.b1 = args.btn==1;
	uiState.b2 = args.btn==2;
	uiState.b3 = args.btn==3;
	for(int i=0; i<AssetManager::GetInstance()->GetNumSliders(); i++) {
		Point2D pos(args.x, uiState.screenHeight-args.y);
		if(AssetManager::GetInstance()->GetSlider(i).IsOnHotspot(i, uiState.screenWidth, uiState.screenHeight, pos)) {
			AssetManager::GetInstance()->GetSlider(i).PickUp();
			break;
		}
	}
}

void OnMouseBtnUp(MouseBtnUpArgs args)
{
	uiState.b1 = args.btn==1;
	uiState.b2 = args.btn==2;
	uiState.b3 = args.btn==3;
	Slider* pickedUp = Slider::GetPickedUp();
	if(pickedUp) pickedUp->PutDown();
}

bool leftPressed = false;
bool rightPressed = false;
bool upPressed = false;
bool downPressed = false;

void OnTimerTick(TimerTickArgs args)
{
	// Spawn a new enemy
	if(ticksTilSpawn <=0) {
		s_enemyList.AddEnemy(Point3D(RandInt(-20, 20), RandInt(-10, 10), RandInt(100, 150)));
		ticksTilSpawn = 10 + ((1-s_spawnfreq) * RandInt(200, 500));
	}

	// Update sprites
	s_missileList.Update(s_cameraPos, s_cameraRot, 100, !s_paused);
	s_enemyList.Update(s_cameraPos, s_cameraRot, s_missileList, s_explosions, s_score, !s_paused);
	for(std::list<Explosion>::iterator iter = s_explosions.begin(); iter!=s_explosions.end(); iter++) {
		iter->Update();
	}
	s_explosions.remove_if(Explosion::IsDone);
	ticksTilSpawn--;
	s_frameCounter++;

	if(s_paused) return;

	const float rotIncr = 1;
	if(upPressed) {
		s_player.m_dir[0] -= rotIncr;
		//s_cameraRot = rotationDeg(rotIncr/2, 'x') * s_cameraRot;
		s_player.CommitRotation();
	}
	if(downPressed) {
		s_player.m_dir[0] += rotIncr;
		//s_cameraRot = rotationDeg(-rotIncr/2, 'x') * s_cameraRot;
		s_player.CommitRotation();
	}
	if(leftPressed) {
		s_player.m_dir[1] += rotIncr;
		//s_cameraRot = rotationDeg(-rotIncr/2, 'y') * s_cameraRot;
		s_player.CommitRotation();
	}
	if(rightPressed) {
		s_player.m_dir[1] -= rotIncr;
		//s_cameraRot = rotationDeg(rotIncr/2, 'y') * s_cameraRot;
		s_player.CommitRotation();
	}
	// blend camera rotation with rotation of ship
	s_cameraRot = s_cameraRot.WeightedMult(s_player.m_rotTrans.invert(), 0.1);
}

void OnKeyDown(KeyDownArgs args)
{
	if(args.key == 's' || args.key == 'S') {
		s_show_ui = !s_show_ui;
	}
	else if(args.key == 'z' || args.key == 'Z') { // fire
		s_missileList.AddMissile(s_player.m_pos, s_player.m_rotTrans * Vector3D(0, 0, 1), s_player.m_rotTrans);
		s_missileList.Update(s_cameraPos, s_cameraRot, 100, true);
		AssetManager::GetInstance()->GetSongPlayer()->Play();
	}
	else if(args.key == 'i' || args.key == 'I') { // nose up
		upPressed = true;
	}
	else if(args.key == 'j' || args.key == 'J') { // left
		leftPressed = true;
	}
	else if(args.key == 'k' || args.key == 'K') { // nose down
		downPressed = true;
	}
	else if(args.key == 'l' || args.key == 'L') { // right
		rightPressed = true;
	}
	else if(args.key == 'r' || args.key == 'R') {
		// reset camera
		//s_currentScene.m_cameraRot = Matrix4x4();
		s_cameraRot = Matrix4x4();
		//s_currentScene.m_cameraTrans = Vector3D();
	}
	else if(args.key == 'p' || args.key == 'P') {
		s_paused = !s_paused;
	}
	else if(args.key == 'e' || args.key == 'E') {
		Explosion e;
		e.Create(Point3D(0, 0, 20));
		s_explosions.push_back(e);
	}
}

void OnKeyUp(KeyUpArgs args)
{
	if(args.key == 'i' || args.key == 'I') { // nose up
		upPressed = false;
	}
	else if(args.key == 'j' || args.key == 'J') { // left
		leftPressed = false;
	}
	else if(args.key == 'k' || args.key == 'K') { // nose down
		downPressed = false;
	}
	else if(args.key == 'l' || args.key == 'L') { // right
		rightPressed = false;
	}
}

void OnScrollDown(ScrollDownArgs args)
{
	s_cameraPos += Vector3D(0, 0, -20);
}

void OnScrollUp(ScrollUpArgs args)
{
	s_cameraPos += Vector3D(0, 0, 20);
}

// 
//  Code below is from http://nehe.gamedev.net/data/lessons/lesson.asp?lesson=45
bool IsExtensionSupported( char* szTargetExtension )
{
	const unsigned char *pszExtensions = NULL;
	const unsigned char *pszStart;
	unsigned char *pszWhere, *pszTerminator;

        // Extension names should not have spaces
	pszWhere = (unsigned char *) strchr( szTargetExtension, ' ' );
	if( pszWhere || *szTargetExtension == '\0' )
		return false;

        // Get Extensions String
	pszExtensions = glGetString( GL_EXTENSIONS );

        // Search The Extensions String For An Exact Copy
	pszStart = pszExtensions;
	for(;;)
	{
		pszWhere = (unsigned char *) strstr( (const char *) pszStart, szTargetExtension );
		if( !pszWhere )
			break;
		pszTerminator = pszWhere + strlen( szTargetExtension );
		if( pszWhere == pszStart || *( pszWhere - 1 ) == ' ' )
			if( *pszTerminator == ' ' || *pszTerminator == '\0' )
				return true;
		pszStart = pszTerminator;
	}
	return false;
}
// end code fragment