#include "appwindow.hpp"
#include "UI.h"

AppWindow::AppWindow()
{
  set_title("A5");

  // A utility class for constructing things that go into menus, which
  // we'll set up next.
  using Gtk::Menu_Helpers::MenuElem;
  
  // Set up the application menu
  // The slot we use here just causes AppWindow::hide() on this,
  // which shuts down the application.
  m_menu_app.items().push_back(MenuElem("_Quit", Gtk::AccelKey("q"),
    sigc::mem_fun(*this, &AppWindow::hide)));
  

  // Set up the menu bar
  m_menubar.items().push_back(Gtk::Menu_Helpers::MenuElem("_Application", m_menu_app));
    
  // Pack in our widgets
  
  // First add the vertical box as our single "top" widget
  add(m_vbox);

  // Put the menubar on the top, and make it as small as possible
  m_vbox.pack_start(m_menubar, Gtk::PACK_SHRINK);

  // Put the viewer below the menubar. pack_start "grows" the widget
  // by default, so it'll take up the rest of the window.
  m_viewer.set_size_request(800, 600);
  m_vbox.pack_start(m_viewer);
  
  const Glib::RefPtr<Glib::TimeoutSource> timeout_source = Glib::TimeoutSource::create ( 33 ); //from glibmm docs
  timeout_source->connect ( sigc::mem_fun ( *this, &AppWindow::timer_handler ) );
  timeout_source->attach ( Glib::MainContext::get_default() );
  
  signal_key_press_event().connect(sigc::mem_fun(*this, &AppWindow::on_key_down)); 
  signal_key_release_event().connect(sigc::mem_fun(*this, &AppWindow::on_key_up)); 


  show_all();
}

bool AppWindow::timer_handler()
{
	TimerTickArgs args;
	OnTimerTick(args);
	m_viewer.invalidate();
	
	return true;
}

bool AppWindow::on_key_down(GdkEventKey* key)
{
	KeyDownArgs args;
	args.key = key->string[0];
	OnKeyDown(args);
}

bool AppWindow::on_key_up(GdkEventKey* key)
{
	KeyUpArgs args;
	args.key = key->string[0];
	OnKeyUp(args);
}