// The code below was taken from /u/gr/cs488/demo/trackball.

#include "stdafx.h"
#include <math.h>
#include "trackball.h"

/*******************************************************
 * 
 * void vCalcRotVec(double fNewX, double fNewY, 
 *                  double fOldX, double fOldY,
 *                  double fDiameter,
 *                  double *fVecX, double *fVecY, double *fVecZ);
 *
 *    Calculates a rotation vector based on mouse motion over
 *    a virtual trackball.
 *
 *    The fNew and fOld mouse positions
 *    should be in 'trackball' space. That is, they have been
 *    transformed into a coordinate system centered at the middle
 *    of the trackball. fNew, fOld, and fDiameter must all be specified
 *    in the same units (pixels for example).
 *
 * Parameters: fNewX, fNewY - New mouse position in trackball space.
 *                            This is the second point along direction
 *                            of rotation.
 *             fOldX, fOldY - Old mouse position in trackball space.
 *                            This is the first point along direction
 *                            of rotation.
 *             fDiameter - Diameter of the trackball. This should
 *                         be specified in the same units as fNew and fOld.
 *                         (ie, usually pixels if fNew and fOld are transformed
 *                         mouse positions)
 *             fVec - The output rotation vector. The length of the vector
 *                    is proportional to the angle of rotation.
 *
 *******************************************************/
void vCalcRotVec(double fNewX, double fNewY,
                 double fOldX, double fOldY,
                 double fDiameter,
                 double *fVecX, double *fVecY, double *fVecZ) {
                   long  nXOrigin, nYOrigin;
                   double fNewVecX, fNewVecY, fNewVecZ,        /* Vector corresponding to new mouse location */
                   fOldVecX, fOldVecY, fOldVecZ,        /* Vector corresponding to old mouse location */
                   fLength;

   /* Vector pointing from center of virtual trackball to
                   * new mouse position
   */
                   fNewVecX    = fNewX * 2.0 / fDiameter;
                   fNewVecY    = fNewY * 2.0 / fDiameter;
                   fNewVecZ    = (1.0 - fNewVecX * fNewVecX - fNewVecY * fNewVecY);

   /* If the Z component is less than 0, the mouse point
                   * falls outside of the trackball which is interpreted
                   * as rotation about the Z axis.
   */
                   if (fNewVecZ < 0.0) {
                     fLength = sqrt(1.0 - fNewVecZ);
                     fNewVecZ  = 0.0;
                     fNewVecX /= fLength;
                     fNewVecY /= fLength;
                   } else {
                     fNewVecZ = sqrt(fNewVecZ);
                   }

   /* Vector pointing from center of virtual trackball to
                   * old mouse position
   */
                   fOldVecX    = fOldX * 2.0 / fDiameter;
                   fOldVecY    = fOldY * 2.0 / fDiameter;
                   fOldVecZ    = (1.0 - fOldVecX * fOldVecX - fOldVecY * fOldVecY);
 
   /* If the Z component is less than 0, the mouse point
                   * falls outside of the trackball which is interpreted
                   * as rotation about the Z axis.
   */
                   if (fOldVecZ < 0.0) {
                     fLength = sqrt(1.0 - fOldVecZ);
                     fOldVecZ  = 0.0;
                     fOldVecX /= fLength;
                     fOldVecY /= fLength;
                   } else {
                     fOldVecZ = sqrt(fOldVecZ);
                   }

   /* Generate rotation vector by calculating cross product:
                   * 
                   * fOldVec x fNewVec.
                   * 
                   * The rotation vector is the axis of rotation
                   * and is non-unit length since the length of a crossproduct
                   * is related to the angle between fOldVec and fNewVec which we need
                   * in order to perform the rotation.
   */
                   *fVecX = fOldVecY * fNewVecZ - fNewVecY * fOldVecZ;
                   *fVecY = fOldVecZ * fNewVecX - fNewVecZ * fOldVecX;
                   *fVecZ = fOldVecX * fNewVecY - fNewVecX * fOldVecY;
                 }

/*******************************************************
                 * void vAxisRotMatrix(double fVecX, double fVecY, double fVecZ, Matrix mNewMat)
                 *    
                 *    Calculate the rotation matrix for rotation about an arbitrary axis.
                 *    
                 *    The axis of rotation is specified by (fVecX,fVecY,fVecZ). The length
                 *    of the vector is the amount to rotate by.
                 *
                 * Parameters: fVecX,fVecY,fVecZ - Axis of rotation
                 *             mNewMat - Output matrix of rotation in column-major format
                 *                       (ie, translation components are in column 3 on rows
                 *                       0,1, and 2).
                 *
 *******************************************************/
                 void vAxisRotMatrix(double fVecX, double fVecY, double fVecZ, Matrix4x4* mNewMat) {
                   double fRadians, fInvLength, fNewVecX, fNewVecY, fNewVecZ;

    /* Find the length of the vector which is the angle of rotation
                   * (in radians)
    */
                   fRadians = sqrt(fVecX * fVecX + fVecY * fVecY + fVecZ * fVecZ);

                   /* If the vector has zero length - return the identity matrix */
                   if (fRadians > -0.000001 && fRadians < 0.000001) {
                     *mNewMat = Matrix4x4();
                     return;
                   }

    /* Normalize the rotation vector now in preparation for making
                   * rotation matrix. 
    */
                   fInvLength = 1 / fRadians;
                   fNewVecX   = fVecX * fInvLength;
                   fNewVecY   = fVecY * fInvLength;
                   fNewVecZ   = fVecZ * fInvLength;

                   /* Create the arbitrary axis rotation matrix */
                   double dSinAlpha = sin(fRadians);
                   double dCosAlpha = cos(fRadians);
                   double dT = 1 - dCosAlpha;
                   
                   mNewMat->set_cell(0, 0, dCosAlpha + fNewVecX*fNewVecX*dT);
                   mNewMat->set_cell(1, 0, fNewVecX*fNewVecY*dT + fNewVecZ*dSinAlpha);
                   mNewMat->set_cell(2, 0, fNewVecX*fNewVecZ*dT - fNewVecY*dSinAlpha);
                   mNewMat->set_cell(3, 0, 0);
                   
                   mNewMat->set_cell(0, 1, fNewVecX*fNewVecY*dT - dSinAlpha*fNewVecZ);
                   mNewMat->set_cell(1, 1, dCosAlpha + fNewVecY*fNewVecY*dT);
                   mNewMat->set_cell(2, 1, fNewVecY*fNewVecZ*dT + dSinAlpha*fNewVecX);
                   mNewMat->set_cell(3, 1, 0);
                   
                   mNewMat->set_cell(0, 2, fNewVecZ*fNewVecX*dT + dSinAlpha*fNewVecY);
                   mNewMat->set_cell(1, 2, fNewVecZ*fNewVecY*dT - dSinAlpha*fNewVecX);
                   mNewMat->set_cell(2, 2, dCosAlpha + fNewVecZ*fNewVecZ*dT);
                   mNewMat->set_cell(3, 2, 0);
                   
                   mNewMat->set_cell(0, 3, 0);
                   mNewMat->set_cell(1, 3, 0);
                   mNewMat->set_cell(2, 3, 0);
                   mNewMat->set_cell(3, 3, 1);
                 }

 
