#include "stdafx.h"
#include "transform.h"

// This code is taken from my handin of assignment 2.

Matrix4x4 rotationDeg(double angle, char axis)
{
  angle = angle*M_PI/180.0; //convert to radians
  return rotationRad(angle, axis);
}


Matrix4x4 rotationRad( double angle, char axis )
{
	Matrix4x4 r;
	if(axis == 'x') {
		r[0][0] = 1; // row 1
		r[0][1] = 0;
		r[0][2] = 0;
		r[0][3] = 0;

		r[1][0] = 0; // row 2
		r[1][1] = cos(angle);
		r[1][2] = -sin(angle);
		r[1][3] = 0;

		r[2][0] = 0; // row 3
		r[2][1] = sin(angle);
		r[2][2] = cos(angle);
		r[2][3] = 0;

		r[3][0] = 0; // row 4
		r[3][1] = 0;
		r[3][2] = 0;
		r[3][3] = 1;
	}
	else if(axis == 'y') {
		r[0][0] = cos(angle); // row 1
		r[0][1] = 0;
		r[0][2] = sin(angle);
		r[0][3] = 0;

		r[1][0] = 0; // row 2
		r[1][1] = 1;
		r[1][2] = 0;
		r[1][3] = 0;

		r[2][0] = -sin(angle); // row 3
		r[2][1] = 0;
		r[2][2] = cos(angle);
		r[2][3] = 0;

		r[3][0] = 0; // row 4
		r[3][1] = 0;
		r[3][2] = 0;
		r[3][3] = 1;
	}
	else if(axis == 'z') {
		r[0][0] = cos(angle); // row 1
		r[0][1] = -sin(angle);
		r[0][2] = 0;
		r[0][3] = 0;

		r[1][0] = sin(angle); // row 2
		r[1][1] = cos(angle);
		r[1][2] = 0;
		r[1][3] = 0;

		r[2][0] = 0; // row 3
		r[2][1] = 0;
		r[2][2] = 1;
		r[2][3] = 0;

		r[3][0] = 0; // row 4
		r[3][1] = 0;
		r[3][2] = 0;
		r[3][3] = 1;
	}
	return r;

}

// Return a matrix to represent a displacement of the given vector.
Matrix4x4 translation(const Vector3D& displacement)
{  
  Matrix4x4 t;
  t[0][0] = 1; // row 1
  t[0][1] = 0;
  t[0][2] = 0;
  t[0][3] = displacement[0];
    
  t[1][0] = 0; // row 2
  t[1][1] = 1;
  t[1][2] = 0;
  t[1][3] = displacement[1];
    
  t[2][0] = 0; // row 3
  t[2][1] = 0;
  t[2][2] = 1;
  t[2][3] = displacement[2];
    
  t[3][0] = 0; // row 4
  t[3][1] = 0;
  t[3][2] = 0;
  t[3][3] = 1;
  return t;
}

// Return a matrix to represent a nonuniform scale with the given factors.
Matrix4x4 scaling(const Vector3D& scale)
{  
  Matrix4x4 s;
  s[0][0] = scale[0]; // row 1
  s[0][1] = 0;
  s[0][2] = 0;
  s[0][3] = 0;
    
  s[1][0] = 0; // row 2
  s[1][1] = scale[1];
  s[1][2] = 0;
  s[1][3] = 0;
    
  s[2][0] = 0; // row 3
  s[2][1] = 0;
  s[2][2] = scale[2];
  s[2][3] = 0;
    
  s[3][0] = 0; // row 4
  s[3][1] = 0;
  s[3][2] = 0;
  s[3][3] = 1;
  return s;
}

// Taken from OpenGL docs.
 Matrix4x4 perspective(double fov, double aspect, double zNear, double zFar)
{
  Matrix4x4 p;
  fov = fov * M_PI / 180; //convert fov to radians
  double f = 1.0 / tan(fov/2.0);
 
  p[0][0] = f / aspect; // row 1
  p[0][1] = 0;
  p[0][2] = 0;
  p[0][3] = 0;
 
  p[1][0] = 0; // row 2
  p[1][1] = f;
  p[1][2] = 0;
  p[1][3] = 0;
 
  p[2][0] = 0; // row 3
  p[2][1] = 0;
  p[2][2] = (zFar + zNear) / (zNear - zFar);
  p[2][3] = (2*zFar*zNear) / (zNear - zFar);
 
  p[3][0] = 0; // row 4
  p[3][1] = 0;
  p[3][2] = -1;
  p[3][3] = 0;
 
  return p;
}

// Taken from OpenGL docs.
Matrix4x4 lookAt(Vector3D view, Vector3D up)
{	
	view.normalize();
	up.normalize();
	Vector3D s = view.cross(up);
	Vector3D u = s.cross(view);
	
	Matrix4x4 m;
	m[0][0] = s[0];
	m[0][1] = s[1];
	m[0][2] = s[2];
	m[0][3] = 0;
    
	m[1][0] = u[0];
	m[1][1] = u[1];
	m[1][2] = u[2];
	m[1][3] = 0;
    
	m[2][0] = -view[0];
	m[2][1] = -view[1];
	m[2][2] = -view[2];
	m[2][3] = 0;
    
	m[3][0] = 0;
	m[3][1] = 0;
	m[3][2] = 0;
	m[3][3] = 1;
	
	return m;	
}
